# embedding_in_qt5.py --- Simple Qt5 application embedding matplotlib canvases
#
# Copyright (C) 2005 Florent Rougon
#               2006 Darren Dale
#               2015 Jens H Nielsen
#
# This file is an example program for matplotlib. It may be used and
# modified with no restriction; raw copies as well as modified versions
# may be distributed without limitation.

from __future__ import unicode_literals

import sys
import os
import matplotlib

matplotlib.use('Qt5Agg')
import matplotlib.pyplot
import numpy as np
from PyQt5 import QtCore, QtWidgets
from PyQt5.QtWidgets import QWidget
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5 import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure


# ------------------------------------------------------------------------------
#           class QMatplotlib_Widget
# ------------------------------------------------------------------------------

class QMatplotlib_Widget:
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""

    # ------------------------------------------------------------------------------
    #          MyMplCanvas::__init__
    # ------------------------------------------------------------------------------

    def __init__(self, parent: QWidget):

        self.parent = parent
        self.width = parent.width()
        self.height = parent.height()
        self.dpi = 0.5 * (parent.logicalDpiX() + parent.logicalDpiY())
        self.figure = Figure()  # Create a figure
        layout = QtWidgets.QVBoxLayout(parent)  # Create a horizontal layout
        self.canvas = FigureCanvas(self.figure)  # Create the canvas (which is
        self.canvas.draw()
        self.toolbar = NavigationToolbar(self.canvas, parent, coordinates=True)
        #        self.toolbar.setOrientation(QtCore.Qt.Vertical)

        layout.addWidget(self.toolbar)
        layout.addWidget(self.canvas)
        layout.setSpacing(0)
        result = parent.setLayout(layout)

    # ------------------------------------------------------------------------------
    #           close_plot
    # ------------------------------------------------------------------------------

    def close_plot(self):
        oldlayout = self.parent.layout()
        if (oldlayout is not None):
            oldlayout.deleteLater()

        # matplotlib.pyplot.close(self.figure)
        self.figure.clf()
        self.canvas.deleteLater()
        self.toolbar.deleteLater()
        self.figure = None
        self.canvas = None
        self.figure = None

    # ------------------------------------------------------------------------------
    #           create_axes
    # ------------------------------------------------------------------------------

    def create_axes(self, numsubplotrows, numsubplotcols, onebased_subplotindex, exactpos=None):

        axes = self.figure.add_subplot(numsubplotrows, numsubplotcols, onebased_subplotindex)
        if (exactpos is not None):
            axes.set_position(exactpos)
        return axes

    # ------------------------------------------------------------------------------
    #           close
    # ------------------------------------------------------------------------------

    def close(self):
        self.figure.clf()
