from .yaml_configuration import Yaml_Configuration
from datetime import datetime
import os
import numpy as np

"""
This class stores the hk data received by the GSE in text files. The data is stored in a directory determined by the
gse_logger.yaml. This directory is sub divided into different days and within each day there is a HK_Data folder. 
Each day's HK_Data folder has a '_FIELDS.txt' file which maintains a list of all hk variables seen
that day. Each hk variable then has its own text file named after itself which holds the values timestamped by 
microseconds since January 1, 1970 which correspond to the timestamp in the hk line it came from.
"""


class hk_logger(object):
    def __init__(self) -> None:
        self.hk_logger_settings: Yaml_Configuration = Yaml_Configuration('gse_logger.yaml')
        self.__current_day_folder_path__ = None
        self.__link_current_day_folder__()

    def __link_current_day_folder__(self) -> None:
        current_date = datetime.now().strftime('%Y-%m-%d')
        self.__current_day_folder_path__ = self.hk_logger_settings.configuration['gse_logger_path'] + os.sep + current_date + os.sep + 'HK_Data'
        if not os.path.exists(self.__current_day_folder_path__):
            os.makedirs(self.__current_day_folder_path__)

    def update_fields_txt(self, name_and_type: tuple) -> None:
        # If fields.txt does not exist for today then make it.
        fields_path = self.__current_day_folder_path__ + os.sep + '_FIELDS.txt'
        if not os.path.exists(fields_path):
            f = open(fields_path, 'w+')
            f.close()

        # Check if field is already listed
        with open(fields_path, 'r') as f:
            content = f.readlines()
            field_there = np.any([name_and_type[0] in x for x in content])

        # If field is not there add it
        if not field_there:
            with open(fields_path, 'a') as f:
                f.write(name_and_type[0] + ':' + name_and_type[1] + '\n')

    def log_value(self, name, value, timestamp):
        # If this variables file does not exist for today then make it.
        variable_path = self.__current_day_folder_path__ + os.sep + name + '.txt'
        if not os.path.exists(variable_path):
            f = open(variable_path, 'w+')
            f.close()

        # make timestamp into microseconds from January 1, 1970
        t = datetime.strptime(timestamp, '%Y-%m-%d %H:%M:%S.%f')
        microseconds = t.timestamp() * 1E6

        # Append data to file
        with open(variable_path, 'a') as f:
            f.write(str(microseconds) + ',' + value.strip() + '\n')
