from typing import Dict, Any, List, Callable
import datetime
import re
import os.path
import os
import logging
from enum import IntEnum
from PyQt5 import QtCore, QtWidgets
from .application import ArgolGSE_QtUI_Application
from .network.udp_listener import UDP_Listener
from .yaml_configuration import Yaml_Configuration
from .widgets.tab_widget_manager import Tab_Widget_Manager
from .widgets.trafficlight import TrafficLight
from .widgets.sensor_variable_widgets import SensorVariableUI_String


# ------------------------------------------------------------------------------
#           CMD_Listener
# ------------------------------------------------------------------------------

class LOG_Listener:

    def __init__(self, qtapp: ArgolGSE_QtUI_Application, tab_manager: Tab_Widget_Manager, config_dir: str = None):

        networkconfig = Yaml_Configuration('network.yaml', basedir=config_dir)
        guiconfig = Yaml_Configuration('logger_gui.yaml', basedir=config_dir)
        guilogconfig = Yaml_Configuration('gse_logger.yaml', basedir=config_dir)
        tabiconname = guiconfig.configuration['LOGGER']['tabicon']
        tokens = tabiconname.split(':')

        self.tabiconname: str = tokens[0]  # The name of the tab whose icon we will control (may be None)
        self.tabiconsection: str = tokens[1]  # The section of the tab icon status that this variable is compared in
        self.tabstatus: TrafficLight = TrafficLight.Green
        self.tab_manager: Tab_Widget_Manager = tab_manager
        self.log_udp_port: SensorVariableUI_String = SensorVariableUI_String('LOG_UDP_PORT', 'str', qtapp, tab_manager, None, options={'tabicon': "voltages_currents:HK"})
#        self.log_udp_port:   QtWidgets.QLabel        = getattr( qtapp.ui,'LOG_UDP_PORT')
        self.firmware_ip: QtWidgets.QLabel = getattr(qtapp.ui, 'FIRMWARE_IP_ADDRESS')
        self.logwidget: QtWidgets.QTextEdit = getattr(qtapp.ui, guiconfig.configuration['LOGGER']['widgetname'])  # Fetch the Qt Designer widget used for the log display
        self.udp: UDP_Listener = UDP_Listener(qtapp, networkconfig.configuration['Network'], networkconfig.configuration['LOGGER'])
        self.specialmessage_router: Dict[str, Callable] = {}
        self.log_basedirectory: str = guilogconfig.configuration['gse_logger_path']
        self.log_basedirectory = os.path.normpath(self.log_basedirectory)

        self.udp.add_listener(self.log_callback)

        self.log_udp_port.set_value(str(self.udp.udp_port_inst))
        if (self.firmware_ip is not None):
            self.firmware_ip.setText(self.firmware_ip_address())

    # ------------------------------------------------------------------------------
    #           firmware_ip_address
    # ------------------------------------------------------------------------------

    def firmware_ip_address(self):
        return self.udp.last_ip_address_recvd

    # ------------------------------------------------------------------------------
    #           register_special_callback
    # ------------------------------------------------------------------------------

    def register_special_callback(self, special_type: str, func: Callable):

        self.specialmessage_router[special_type] = func

    # ------------------------------------------------------------------------------
    #           hk_callback
    # ------------------------------------------------------------------------------

    def log_callback(self, log_packet: str):

        if (log_packet is not None and len(log_packet) > 0):  # None and sero length occur when there is a timeout or reception error
            if (self.firmware_ip is not None):
                self.firmware_ip.setText(self.firmware_ip_address())

            m = re.match(r'(.+)##([A-Z]+)##(.+)', log_packet)  # is this a special LOG message, ARCHIVE, STATS, SAMPLED etc.
            if (m is not None):  # if it is is
                type = m.group(2)  # then get the type of specila message
                leader = m.group(1)  # get the message leader
                mes = m.group(3)  # and the main message
                func = self.specialmessage_router.get(type)  # and execute the function
                if func is not None:
                    func(leader, mes)
                else:
                    pass
                    # print('Ignoring SPECIAL LOG:', log_packet)
            else:
                tnow = datetime.datetime.utcnow()
                tmes = tnow.strftime("%Y%m%d")
                basefolder = self.log_basedirectory + os.sep + tmes
                basefolder = os.path.normpath(basefolder)
                os.makedirs(basefolder, exist_ok=True)
                filename = basefolder + os.sep + "gselog_" + tmes + ".txt"
                f = open(filename, 'a+t')
                f.write("%s\n" % (log_packet,))
                f.close()

                if (log_packet.find('WARNING') == 0):
                    if (self.tabstatus < TrafficLight.Orange):
                        self.tabstatus = TrafficLight.Orange
                if (log_packet.find('ERROR') == 0):
                    self.tabstatus = TrafficLight.Orange
                self.tab_manager.update_tab_status(self.tabiconname, self.tabiconsection, self.tabstatus)
                self.logwidget.append(log_packet)
