from typing import Dict, List, Any, Callable
from PyQt5 import QtCore, QtWidgets
import threading
import logging
import socket
from .udp_listener import mac_address_to_ip_and_broadcast


# ------------------------------------------------------------------------------
#           UDP_Listener
# ------------------------------------------------------------------------------

class UDP_Transmit():
    """
    A class that transmits messages to a UDP ports
    """

    # ------------------------------------------------------------------------------
    #          SHOWSGE_TemperatureMonitorThread:: __init__
    # ------------------------------------------------------------------------------

    def __init__(self, networkconfig: Dict[str, Any], config: Dict[str, Any]):

        ipaddress = mac_address_to_ip_and_broadcast(networkconfig['gse_interface'])
        if (ipaddress is None):
            print("GSE network [{}]is not available, disconnected or not yet available.".format(
                networkconfig['gse_interface']))
            raise Exception("GSE network [{}]is not available, disconnected or not yet available. If you see this message in lab bench configuration please turn on ALI flight computer and restart this program".format(
                networkconfig['gse_interface']))
        self.udp_ipaddress: str = ipaddress
        self.udp_destination: str = networkconfig['instrument_ip']  # 10.65.0.29' #broadcast_address
        self.udp_port_gse: int = int(config['udp_port_gse'])
        self.udp_port_inst: int = int(config['udp_port_inst'])
        self.lock: threading.Lock = threading.Lock()

        self.serverSocket: socket.socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)  # Create a UDP socket, Notice the use of SOCK_DGRAM for UDP packets
        self.serverSocket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        if (self.udp_destination == '255.255.255.255'):
            self.serverSocket.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
            print('Broadcasting UDP packets to instrument')
        print('Binding UDP outbound connection from {}:{} sending to {}:{}'.format(self.udp_ipaddress, self.udp_port_gse, self.udp_destination, self.udp_port_inst))
        self.serverSocket.bind((self.udp_ipaddress, self.udp_port_gse))

    def send(self, message: str):
        self.lock.acquire()
        self.serverSocket.sendto(message.encode('utf-8'), (self.udp_destination, self.udp_port_inst))
        self.lock.release()
