from typing import Dict, Any, Tuple, List, Union
import numpy as np
from matplotlib.axes import Axes
from matplotlib.image import AxesImage
from ..application import ArgolGSE_QtUI_Application
from .matplotlib_in_qtwidget import QMatplotlib_Widget


# ------------------------------------------------------------------------------
#           ImageDisplayGUI
# ------------------------------------------------------------------------------

class ImageDisplayGUI:

    def __init__(self, globalobject: ArgolGSE_QtUI_Application,
                 IMAGE_WIDGET_NAME: str = 'IMAGE_DISPLAY_BOX',
                 HISTOGRAM_DISPLAY_NAME: str = 'HISTOGRAM_DISPLAY_BOX',
                 SIDE_DISPLAY_NAME: str = 'SIDE_DISPLAY_BOX',
                 UTC_DISPLAY_NAME: str = 'IMG_UTC_STR'):
        ui = globalobject.ui
        IMAGE_DISPLAY_BOX = getattr(ui, IMAGE_WIDGET_NAME)
        HISTOGRAM_DISPLAY_BOX = getattr(ui, HISTOGRAM_DISPLAY_NAME)
        SIDE_DISPLAY_BOX = getattr(ui, SIDE_DISPLAY_NAME)
        self.IMG_UTC_STR = getattr(ui, UTC_DISPLAY_NAME)

        self.plotwindow: QMatplotlib_Widget = QMatplotlib_Widget(IMAGE_DISPLAY_BOX)
        self.histowindow: QMatplotlib_Widget = QMatplotlib_Widget(HISTOGRAM_DISPLAY_BOX)
        self.sidewindow: QMatplotlib_Widget = QMatplotlib_Widget(SIDE_DISPLAY_BOX)
        self.axes: Axes = self.plotwindow.create_axes(1, 1, 1)
        self.imageobject: AxesImage = None
        self.image: np.ndarray = None

    # ------------------------------------------------------------------------------
    #           update_image
    # ------------------------------------------------------------------------------

    def update_image(self, image: np.ndarray, utcstr: str, exactpos=None):
        self.image = image
        self.plot_image(exactpos=exactpos)
        self.plot_histo()
        self.plot_sideplot()
        self.IMG_UTC_STR.setText(utcstr)

    # ------------------------------------------------------------------------------
    #           plot_image
    # ------------------------------------------------------------------------------

    def plot_image(self, exactpos=None):
        p2 = np.percentile(self.image, 2.0)
        p98 = np.percentile(self.image, 98.0)
        self.plotwindow.figure.clf()
        self.axes = self.plotwindow.create_axes(1, 1, 1, exactpos=exactpos)
        self.imageobject = self.axes.imshow(self.image, origin='lower', aspect='auto')
        self.imageobject.set_clim((p2, p98))
        self.plotwindow.figure.colorbar(self.imageobject)
        self.plotwindow.canvas.draw()

    # ------------------------------------------------------------------------------
    #           plot_histo
    # ------------------------------------------------------------------------------

    def plot_histo(self):
        self.histowindow.figure.clf()
        bins = np.arange(2000, 16400, 100)
        subimage = self.image[200:490, 12:500]
        axes = self.histowindow.create_axes(1, 1, 1)
        axes.hist(subimage.reshape([-1]), bins=bins)
        self.histowindow.canvas.draw()

    # ------------------------------------------------------------------------------
    #           plot_histo
    # ------------------------------------------------------------------------------

    def plot_sideplot(self):
        self.sidewindow.figure.clf()
        rows = np.arange(200, 490, 1)
        subimage = self.image[200:490, 12:500]
        avg = np.mean(subimage, 1)
        axes = self.sidewindow.create_axes(1, 1, 1)
        axes.plot(avg, rows)
        axes.axhline(292)
        # axes.xlabel('Mean Row Signal')
        # axes.ylabel('Height Row')
        self.sidewindow.canvas.draw()
