from typing import Dict, Any, Tuple, List, Union
import datetime
import math
import os.path
import numpy as np
import matplotlib.axes
from PyQt5 import QtCore, QtGui, QtWidgets
# from .. import mainglobalobject
from ..matplotlib_in_qtwidget import QMatplotlib_Widget
from showapi.level0.owl640_xiphosraw import SHOW_RawData_Owl640Image


# ------------------------------------------------------------------------------
#           SHOWGSE_ImageDisplayGui
# ------------------------------------------------------------------------------

class SHOWGSE_ImageDisplayGui:

    def __init__(self, globalobject: 'mainglobalobject.GSE_MainGlobalObject'):
        self.globalobject = globalobject  # type: 'mainglobalobject.GSE_MainGlobalObject'
        ui = globalobject.gui_app.ui
        self.IMAGE_DISPLAY_BOX = ui.IMAGE_DISPLAY_BOX
        self.HISTOGRAM_DISPLAY_BOX = ui.HISTOGRAM_DISPLAY_BOX
        self.SIDE_DISPLAY_BOX = ui.SIDE_DISPLAY_BOX
        self.IMG_UTC_STR = ui.IMG_UTC_STR
        self.plotwindow = QMatplotlib_Widget(self.IMAGE_DISPLAY_BOX)
        self.histowindow = QMatplotlib_Widget(self.HISTOGRAM_DISPLAY_BOX)
        self.sidewindow = QMatplotlib_Widget(self.SIDE_DISPLAY_BOX)
        self.axes = self.plotwindow.create_axes(1, 1, 1)
        self.image = None
        self.imageobject = None

    # ------------------------------------------------------------------------------
    #           update_image
    # ------------------------------------------------------------------------------

    def update_image(self, filename: str, utcstr: str):
        rawimageobj = SHOW_RawData_Owl640Image(filename)
        self.image = rawimageobj.Image()
        self.plot_image()
        self.plot_histo()
        self.plot_sideplot()
        self.IMG_UTC_STR.setText(utcstr)

    # ------------------------------------------------------------------------------
    #           plot_image
    # ------------------------------------------------------------------------------

    def plot_image(self):
        p2 = np.percentile(self.image, 2.0)
        p98 = np.percentile(self.image, 98.0)
        self.plotwindow.figure.clf()
        self.axes = self.plotwindow.create_axes(1, 1, 1)
        self.imageobject = self.axes.imshow(self.image, origin='lower')
        self.imageobject.set_clim((p2, p98))
        self.plotwindow.figure.colorbar(self.imageobject)
        self.plotwindow.canvas.draw()

    # ------------------------------------------------------------------------------
    #           plot_histo
    # ------------------------------------------------------------------------------

    def plot_histo(self):
        self.histowindow.figure.clf()
        bins = np.arange(2000, 16400, 100)
        subimage = self.image[200:490, 12:500]
        axes = self.histowindow.create_axes(1, 1, 1)
        axes.hist(subimage.reshape([-1]), bins=bins)
        self.histowindow.canvas.draw()

    # ------------------------------------------------------------------------------
    #           plot_histo
    # ------------------------------------------------------------------------------

    def plot_sideplot(self):
        self.sidewindow.figure.clf()
        rows = np.arange(200, 490, 1)
        subimage = self.image[200:490, 12:500]
        avg = np.mean(subimage, 1)
        axes = self.sidewindow.create_axes(1, 1, 1)
        axes.plot(avg, rows)
        axes.axhline(292)
        # axes.xlabel('Mean Row Signal')
        # axes.ylabel('Height Row')
        self.sidewindow.canvas.draw()
