from typing import Dict, Any, Tuple, List, Union, Callable
import logging
import sys

from ..application import ArgolGSE_QtUI_Application
from ..yaml_configuration import Yaml_Configuration
from .trafficlight import TrafficLight
from .sensor_variable_widgets import SensorVariableUI_Base, SensorVariableUI_String, SensorVariableUI_BoundedFloat, SensorVariableUI_Relay, SensorVariableUI_IncreasingInteger, SensorVariableUI_Integer, SensorVariableUI_MaskedBit, SensorVariableUI_String_WithBadValues
from .tab_widget_manager import Tab_Widget_Manager
from ..cmd import CMD_Manager


# -----------------------------------------------------------------------------------------------------------------------
#           class SensorVariableUI_Array
# -----------------------------------------------------------------------------------------------------------------------

class SensorVariableUI_Array:

    def __init__(self, qtapp: ArgolGSE_QtUI_Application,
                 tab_widget_manager: Tab_Widget_Manager,
                 cmd_mgr: CMD_Manager, config_dir=None):

        self._sensor_variables: Dict[str, SensorVariableUI_Base] = {}
        self._hkmapping: Dict[str, SensorVariableUI_Base] = {}  # The array of Sensor Variable widgets# The array of Sensor Variable widgets
        self.tab_widget_manager: Tab_Widget_Manager = tab_widget_manager
        self.cmd_manager: CMD_Manager = cmd_mgr
        self.load_widgets(qtapp, config_dir=config_dir)

    # ------------------------------------------------------------------------------
    #           __iter__
    # ------------------------------------------------------------------------------

    def items(self):
        return self._sensor_variables.items()

    # ------------------------------------------------------------------------------
    #           count
    # ------------------------------------------------------------------------------

    def count(self):
        return len(self._sensor_variables)

    # ------------------------------------------------------------------------------
    #           update_widgets
    # ------------------------------------------------------------------------------

    def load_widgets(self, qtapp: ArgolGSE_QtUI_Application, config_dir=None):
        """
        Loads in the sensor variable widgets defined in file `hkvariables.yaml`. The code is not robust to typing errors
        in hkvariables.yaml, so make sure it is properly formatted

        The sensor Variable class name is taken from the `widget` property and is used to create an instance of that class.
        The creation will catastrophically fail if the class has not been already imported into the
        `sensor_variable_array` module
        """

        variablenames: Yaml_Configuration = Yaml_Configuration('hkvariables.yaml', basedir=config_dir)
        module = sys.modules[__name__]
        for name, options in variablenames.configuration.items():
            type = options.get('widget')
            sensorvariableclass = getattr(module, type)
            entry = sensorvariableclass(name, type, qtapp, self.tab_widget_manager, self.cmd_manager, options)
            assert(entry.sensor_name is not None)
            assert(entry.hk_name is not None)
            self._sensor_variables[entry.sensor_name] = entry
            self._hkmapping[entry.hk_name] = entry

    # ------------------------------------------------------------------------------
    #           find
    # ------------------------------------------------------------------------------

    def find(self, sensor_variable_name: str) -> SensorVariableUI_Base:

        """
        Finds the sensor variable with the specified name. This should be same name as the top level key used in `hkvariables.yaml`
        """

        variable = self._sensor_variables.get(sensor_variable_name)
        return variable

    # ------------------------------------------------------------------------------
    #           find
    # ------------------------------------------------------------------------------

    def find_from_hkname(self, hkname: str) -> SensorVariableUI_Base:

        """
        Finds the sensor variable with the specified name. This should be same name as the top level key used in `hkvariables.yaml`
        """

        variable = self._hkmapping.get(hkname)
        return variable
