import yaml
import os
import os.path
import math
import numpy as np
import pickle
import logging
from typing import Dict, Tuple, List, Union, Any


class Yaml_Configuration:

    def __init__(self, yaml_filename: str, basedir: str = None):

        self.configuration: Dict[str, Any] = self.read_configuration(yaml_filename, basedir)

    # ------------------------------------------------------------------------------
    #           read_configuration
    # ------------------------------------------------------------------------------

    def read_configuration(self, yaml_name: str, basedir: str = None) -> Dict[str, Any]:

        if basedir is None:
            basedir = os.getcwd() + os.sep + "config"
        config = {}  # load in site specific settings
        fullname = os.path.normpath(basedir + os.sep + yaml_name)
        try:
            f = open(fullname, 'rt')
            config = yaml.load(f, Loader=yaml.FullLoader)
            f.close()

        except Exception as inst:
            print(type(inst))  # the exception instance
            print(inst.args)  # arguments stored in .args
            print(inst)  # __str__ allows args to be printed directly,
            config = None
            raise RuntimeError('Error reading configuration file from file %s' % (fullname,))

        return config
